from datetime import date as date_type, datetime, time as time_type
from typing import Optional

import numpy as np
from datasets import load_dataset
from fastapi import HTTPException
import pandas as pd

from source.configs import NDMI_DATASET_ID, PREDICTIONS_DATASET_ID, CUMULATED_DATASET_ID, STD_TIME_FORMAT, \
    IDRO_PREDICTIONS_CONFIGS, IDRO_DATASET_ID, IDRO_DATETIME_FORMAT, STD_DATETIME_FORMAT, NDMI_DATETIME_FORMAT
from source.resources import AcquiredNdmiResource, PredictedNdmiResource, CumulatedPluvioResource, \
    PredictedFloodResource


def get_acquired_ndmi_by_date(date: date_type, token: str):
    formatted_date = date.strftime(STD_TIME_FORMAT)
    try:
        ndmi_dataset = load_dataset(NDMI_DATASET_ID, use_auth_token=token, split="train", name=f"{formatted_date}-exact")
    except Exception as e:
        print("Raised exception:")
        print(str(e))
        raise HTTPException(status_code=404, detail=f"Acquired NDMI for date {date} not found")
    ndmi_image = ndmi_dataset[0]['image']
    gradient_img = _gradient_alg(np.array(ndmi_image))
    gradient_img = gradient_img.squeeze()
    ndmi_timestamp = ndmi_dataset[0]['timestamp']
    return AcquiredNdmiResource(
        timestamp=ndmi_timestamp,
        data=gradient_img.tolist()
    )


def get_predicted_ndmi_by_date(date: date_type, time: time_type, token: str, host: str):
    if date.year < 2023:
        formatted_date = date.strftime(STD_TIME_FORMAT)
    else:
        formatted_date = datetime(date.year, date.month, date.day, time.hour, 0)
        formatted_date = formatted_date.strftime(NDMI_DATETIME_FORMAT)
    try:
        pred_dataset = load_dataset(PREDICTIONS_DATASET_ID, use_auth_token=token, split="train", name=formatted_date)
    except Exception as e:
        print("Raised exception:")
        print(str(e))
        raise HTTPException(status_code=404, detail=f"Predicted NDMI for date {formatted_date} not found")
    pred_image = pred_dataset[0]['image']
    np_pred = np.array(pred_image)
    np_squeezed = np_pred.squeeze()
    pred_ndmi_timestamp = pred_dataset[0]['ndmi_timestamp']
    formatted_timestamp = pred_ndmi_timestamp.strftime(STD_TIME_FORMAT)
    return PredictedNdmiResource(
        timestamp=date,
        data=np_squeezed.tolist(),
        cumulated_pluvio_ref=f"{host}/cumulated-pluvio?date={formatted_date}",
        ndmi_ref=f"{host}/acquired-ndmis?date={formatted_timestamp}"
    )


def get_predicted_flood_by_date(date: datetime, station: Optional[str], token: str, host: str):
    if station:
        configs = [(k, c) for k, c in IDRO_PREDICTIONS_CONFIGS.items() if k == station]
        if not configs:
            raise HTTPException(
                status_code=400,
                detail=f"Station {station} unavailable. Available stations are {list(IDRO_PREDICTIONS_CONFIGS.keys())}"
            )
    else:
        configs = IDRO_PREDICTIONS_CONFIGS.items()
    resources = []
    for k, c in configs:
        try:
            pred_dataset = load_dataset(IDRO_DATASET_ID, use_auth_token=token, split="train", name=c)
        except Exception as e:
            print("Raised exception:")
            print(str(e))
            raise HTTPException(status_code=404, detail=f"Could not load configuration {c} for idro dataset")
        idro_df_data = pred_dataset['idro_df'][0]
        df = pd.DataFrame(idro_df_data)
        filtered_df = df[df['timestamp'] == date.strftime(IDRO_DATETIME_FORMAT)]
        if filtered_df.empty:
            raise HTTPException(status_code=404, detail=f"Date {date} not found for configuration {c} in idro dataset")
        value_df = filtered_df['value']
        value = value_df.values[0]
        formatted_date = date.strftime(STD_DATETIME_FORMAT)
        resources.append(PredictedFloodResource(
            station=k,
            timestamp=date,
            value=value,
            cumulated_pluvio_ref=f"{host}/cumulated-pluvio?date={formatted_date}",
            ndmi_ref=f"{host}/predicted-ndmis?date={formatted_date}"
        ))
    return resources


def get_predicted_flood_by_day(date: date_type, station: Optional[str], token: str, host: str):
    if station:
        configs = [(k, c) for k, c in IDRO_PREDICTIONS_CONFIGS.items() if k == station]
        if not configs:
            raise HTTPException(
                status_code=400,
                detail=f"Station {station} unavailable. Available stations are {list(IDRO_PREDICTIONS_CONFIGS.keys())}"
            )
    else:
        configs = IDRO_PREDICTIONS_CONFIGS.items()
    resources = []
    for k, c in configs:
        try:
            pred_dataset = load_dataset(IDRO_DATASET_ID, use_auth_token=token, split="train", name=c)
        except Exception as e:
            print("Raised exception:")
            print(str(e))
            raise HTTPException(status_code=404, detail=f"Could not load configuration {c} for idro dataset")
        idro_df_data = pred_dataset['idro_df'][0]
        df = pd.DataFrame(idro_df_data)
        filtered_df = df[df['timestamp'].str.contains(date.strftime(STD_TIME_FORMAT))]
        if filtered_df.empty:
            raise HTTPException(status_code=404, detail=f"Date {date} not found for configuration {c} in idro dataset")
        for _, row in filtered_df.iterrows():
            ts = datetime.strptime(row['timestamp'], IDRO_DATETIME_FORMAT)
            value = row['value']
            formatted_date = ts.strftime(STD_DATETIME_FORMAT)
            resources.append(PredictedFloodResource(
                station=k,
                timestamp=ts,
                value=value,
                cumulated_pluvio_ref=f"{host}/cumulated-pluvio?date={formatted_date}",
                ndmi_ref=f"{host}/predicted-ndmis?date={formatted_date}"
            ))
    return resources


def get_cumulated_pluvio_by_date(date: date_type, token: str, host: str):
    formatted_date = date.strftime(STD_TIME_FORMAT)
    try:
        cumulated_dataset = load_dataset(CUMULATED_DATASET_ID, use_auth_token=token, split="train", name=formatted_date)
    except Exception as e:
        print("Raised exception:")
        print(str(e))
        raise HTTPException(status_code=404, detail=f"Cumulated for date {date} not found")
    cumulated_image = cumulated_dataset[0]['image']
    print(f"Type of cumulated image: {type(cumulated_image)}")
    np_cumulated = np.array(cumulated_image)
    np_squeezed = np_cumulated.squeeze()
    related_ndmi_timestamp = cumulated_dataset[0]['ndmi_timestamp']
    formatted_timestamp = related_ndmi_timestamp.strftime(STD_TIME_FORMAT)
    return CumulatedPluvioResource(
        timestamp=date,
        data=np_squeezed.tolist(),
        type="PREDICTED",
        ndmi_ref=f"{host}/acquired-ndmis?date={formatted_timestamp}"
    )


def _gradient_alg(img):
    img = img.reshape(256, 256)
    for i, row in enumerate(img):
        mask = np.isnan(row)

        x = np.arange(len(row))
        y = row[~mask]
        if len(y) > 1:
            gradient = np.gradient(y)
            row[mask] = np.interp(x[mask], x[~mask], gradient)
        elif len(y) == 1:
            row[mask] = np.ones(255)
            print("not enough points to calculate the gradient: ", len(y))
        else:
            row[mask] = np.ones(256)
            print("not enough points to calculate the gradient: ", len(y))
        img[i] = row
    img = img.reshape(256, 256, 1)
    return img
