from datetime import date, time

import numpy as np

from source.dataset_service import get_acquired_ndmi_by_date, get_predicted_ndmi_by_date, get_cumulated_pluvio_by_date
from source.resources import AcquiredNdmiResource, PredictedNdmiResource, CumulatedPluvioResource


def test_get_acquired_ndmi_by_date(mocker):
    # Mocked data
    input_date = date(2023, 1, 10)
    input_token = "your_token"
    ndmi_dataset = [
        {'image': [[0.5, 0.3], [0.7, 0.9]], 'timestamp': date(2023, 1, 10)},
    ]
    mocker.patch('source.dataset_service.load_dataset', return_value=ndmi_dataset)
    mocker.patch('source.dataset_service._gradient_alg', return_value=np.array([[0.2, 0.4], [0.6, 0.8]]))

    # Expected result
    expected_result = AcquiredNdmiResource(
        timestamp="2023-01-10",
        data=[[0.2, 0.4], [0.6, 0.8]]
    )

    # Call the function
    result = get_acquired_ndmi_by_date(input_date, input_token)

    # Assertions
    assert result.timestamp == expected_result.timestamp
    assert result.data == expected_result.data


def test_get_predicted_ndmi_by_date(mocker):
    # Mocked data
    input_date = date(2021, 1, 10)
    input_time = time(10, 10)
    input_token = "your_token"
    input_host = "my-host"
    ndmi_dataset = [
        {'image': [[0.5, 0.3], [0.7, 0.9]], 'ndmi_timestamp': date(2021, 1, 2)},
    ]
    mocker.patch('source.dataset_service.load_dataset', return_value=ndmi_dataset)

    # Expected result
    expected_result = PredictedNdmiResource(
        timestamp="2021-01-10",
        data=[[0.5, 0.3], [0.7, 0.9]],
        cumulated_pluvio_ref=f"{input_host}/cumulated-pluvio?date=20210110",
        ndmi_ref=f"{input_host}/acquired-ndmis?date=20210102"
    )

    # Call the function
    result = get_predicted_ndmi_by_date(input_date, input_time, input_token, input_host)

    # Assertions
    assert result.timestamp == expected_result.timestamp
    assert result.data == expected_result.data
    assert result.cumulated_pluvio_ref == expected_result.cumulated_pluvio_ref
    assert result.ndmi_ref == expected_result.ndmi_ref


def test_get_cumulated_pluvio_by_date(mocker):
    # Mocked data
    input_date = date(2023, 1, 10)
    input_token = "your_token"
    input_host = "my-host"
    ndmi_dataset = [
        {'image': [[0.5, 0.3], [0.7, 0.9]], 'ndmi_timestamp': date(2023, 1, 2)},
    ]
    mocker.patch('source.dataset_service.load_dataset', return_value=ndmi_dataset)

    # Expected result
    expected_result = CumulatedPluvioResource(
        timestamp="2023-01-10",
        data=[[0.5, 0.3], [0.7, 0.9]],
        type="PREDICTED",
        ndmi_ref=f"{input_host}/acquired-ndmis?date=20230102"
    )

    # Call the function
    result = get_cumulated_pluvio_by_date(input_date, input_token, input_host)

    # Assertions
    assert result.timestamp == expected_result.timestamp
    assert result.data == expected_result.data
    assert result.type == expected_result.type
    assert result.ndmi_ref == expected_result.ndmi_ref
